@@ -3,7 +3,11 @@
 from decimal import Decimal
 import datetime
 import operator
-from itertools import izip
+from itertools import izip, groupby
+from sql import Column, Literal
+from sql.aggregate import Sum
+from sql.conditionals import Coalesce
+
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateView, StateAction, StateTransition, \
     Button
@@ -12,7 +16,7 @@ from trytond.tools import reduce_ids
 from trytond.pyson import Eval, PYSONEncoder, Date
 from trytond.transaction import Transaction
 from trytond.pool import Pool
-from trytond.backend import TableHandler
+from trytond import backend
 
 __all__ = ['TypeTemplate', 'Type', 'OpenType', 'AccountTemplate', 'Account',
     'AccountDeferral', 'OpenChartAccountStart', 'OpenChartAccount',
@@ -34,9 +38,7 @@ class TypeTemplate(ModelSQL, ModelView):
             ondelete="RESTRICT")
     childs = fields.One2Many('account.account.type.template', 'parent',
         'Children')
-    sequence = fields.Integer('Sequence',
-        order_field='(%(table)s.sequence IS NULL) %(order)s, '
-        '%(table)s.sequence %(order)s')
+    sequence = fields.Integer('Sequence')
     balance_sheet = fields.Boolean('Balance Sheet')
     income_statement = fields.Boolean('Income Statement')
     display_balance = fields.Selection([
@@ -51,6 +53,7 @@ class TypeTemplate(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
 
@@ -64,6 +67,11 @@ class TypeTemplate(ModelSQL, ModelView):
         super(TypeTemplate, cls).validate(records)
         cls.check_recursion(records, rec_name='name')
 
+    @staticmethod
+    def order_sequence(tables):
+        table, _ = tables[None]
+        return [table.sequence == None, table.sequence]
+
     @staticmethod
     def default_balance_sheet():
         return False
@@ -165,8 +173,6 @@ class Type(ModelSQL, ModelView):
             ('company', '=', Eval('company')),
         ], depends=['company'])
     sequence = fields.Integer('Sequence',
-        order_field='(%(table)s.sequence IS NULL) %(order)s, '
-        '%(table)s.sequence %(order)s',
         help='Use to order the account type')
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
             'get_currency_digits')
@@ -190,6 +196,7 @@ class Type(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
 
@@ -203,6 +210,11 @@ class Type(ModelSQL, ModelView):
         super(Type, cls).validate(types)
         cls.check_recursion(types, rec_name='name')
 
+    @staticmethod
+    def order_sequence(tables):
+        table, _ = tables[None]
+        return [table.sequence == None, table.sequence]
+
     @staticmethod
     def default_balance_sheet():
         return False
@@ -236,9 +248,10 @@ class Type(ModelSQL, ModelView):
 
         accounts = Account.search([
                 ('type', 'in', [t.id for t in childs]),
+                ('kind', '!=', 'view'),
                 ])
         for account in accounts:
-            type_sum[account.type.id] += account.balance
+            type_sum[account.type.id] += (account.debit - account.credit)
 
         for type_ in types:
             childs = cls.search([
@@ -318,10 +331,12 @@ class OpenType(Wizard):
     def do_open_(self, action):
         action['pyson_domain'] = PYSONEncoder().encode([
                 ('type', '=', Transaction().context['active_id']),
+                ('kind', '!=', 'view'),
                 ])
         action['pyson_context'] = PYSONEncoder().encode({
                 'date': Transaction().context.get('date'),
                 'posted': Transaction().context.get('posted'),
+                'cumulate': Transaction().context.get('cumulate'),
                 })
         return action, {}
 
@@ -389,12 +404,10 @@ class AccountTemplate(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        templates = cls.search([
-                ('code',) + clause[1:],
-                ], limit=1)
-        if templates:
-            return [('code',) + clause[1:]]
-        return [(cls._rec_name,) + clause[1:]]
+        return ['OR',
+            ('code',) + tuple(clause[1:]),
+            (cls._rec_name,) + tuple(clause[1:]),
+            ]
 
     def _get_account_value(self, account=None):
         '''
@@ -498,7 +511,7 @@ class AccountTemplate(ModelSQL, ModelView):
             if self.taxes:
                 Account.write([Account(template2account[self.id])], {
                         'taxes': [
-                            ('add', template2tax[x.id]) for x in self.taxes],
+                            ('add', [template2tax[x.id] for x in self.taxes])],
                         })
             template_done.append(self.id)
 
@@ -532,7 +545,9 @@ class Account(ModelSQL, ModelView):
             ('company', '=', Eval('company')),
             ], depends=['kind', 'company'])
     parent = fields.Many2One('account.account', 'Parent', select=True,
-            left="left", right="right", ondelete="RESTRICT")
+        left="left", right="right", ondelete="RESTRICT",
+        domain=[('company', '=', Eval('company'))],
+        depends=['company'])
     left = fields.Integer('Left', required=True, select=True)
     right = fields.Integer('Right', required=True, select=True)
     childs = fields.One2Many('account.account', 'parent', 'Children')
@@ -624,100 +639,48 @@ class Account(ModelSQL, ModelView):
 
     @classmethod
     def get_balance(cls, accounts, name):
-        res = {}
         pool = Pool()
-        Currency = pool.get('currency.currency')
         MoveLine = pool.get('account.move.line')
         FiscalYear = pool.get('account.fiscalyear')
-        Deferral = pool.get('account.account.deferral')
         cursor = Transaction().cursor
+        in_max = cursor.IN_MAX
 
-        query_ids, args_ids = cls.search([
-                ('parent', 'child_of', [a.id for a in accounts]),
-                ], query_string=True)
-        line_query, fiscalyear_ids = MoveLine.query_get()
-        cursor.execute('SELECT a.id, '
-                    'SUM((COALESCE(l.debit, 0) - COALESCE(l.credit, 0))) '
-                'FROM account_account a '
-                    'LEFT JOIN account_move_line l '
-                    'ON (a.id = l.account) '
-                'WHERE a.kind != \'view\' '
-                    'AND a.id IN (' + query_ids + ') '
-                    'AND ' + line_query + ' '
-                    'AND a.active '
-                'GROUP BY a.id', args_ids)
-        account_sum = {}
-        for account_id, sum in cursor.fetchall():
-            # SQLite uses float for SUM
-            if not isinstance(sum, Decimal):
-                sum = Decimal(str(sum))
-            account_sum[account_id] = sum
+        table_a = cls.__table__()
+        table_c = cls.__table__()
+        line = MoveLine.__table__()
+        ids = [a.id for a in accounts]
+        balances = dict((i, 0) for i in ids)
+        line_query, fiscalyear_ids = MoveLine.query_get(line)
+        for i in range(0, len(ids), in_max):
+            sub_ids = ids[i:i + in_max]
+            red_sql = reduce_ids(table_a.id, sub_ids)
+            cursor.execute(*table_a.join(table_c,
+                    condition=(table_c.left >= table_a.left)
+                    & (table_c.right <= table_a.right)
+                    ).join(line, condition=line.account == table_c.id
+                    ).select(
+                    table_a.id,
+                    Sum(Coalesce(line.debit, 0) - Coalesce(line.credit, 0)),
+                    where=red_sql & line_query & table_c.active,
+                    group_by=table_a.id))
+            result = cursor.fetchall()
+            balances.update(dict(result))
 
-        account2company = {}
-        id2company = {}
-        id2account = {}
-        all_accounts = cls.search([('parent', 'child_of',
-                    [a.id for a in accounts])])
-        for account in all_accounts:
-            account2company[account.id] = account.company.id
-            id2company[account.company.id] = account.company
-            id2account[account.id] = account
+        # SQLite uses float for SUM
+        for account_id, balance in balances.iteritems():
+            if isinstance(balance, Decimal):
+                break
+            balances[account_id] = Decimal(str(balance))
 
         for account in accounts:
-            res.setdefault(account.id, Decimal('0.0'))
-            childs = cls.search([
-                    ('parent', 'child_of', [account.id]),
-                    ])
-            company_id = account2company[account.id]
-            to_currency = id2company[company_id].currency
-            for child in childs:
-                child_company_id = account2company[child.id]
-                from_currency = id2company[child_company_id].currency
-                res[account.id] += Currency.compute(from_currency,
-                        account_sum.get(child.id, Decimal('0.0')), to_currency,
-                        round=True)
+            balances[account.id] = account.company.currency.round(
+                balances[account.id])
 
-        youngest_fiscalyear = None
-        for fiscalyear in FiscalYear.browse(fiscalyear_ids):
-            if not youngest_fiscalyear \
-                    or youngest_fiscalyear.start_date > fiscalyear.start_date:
-                youngest_fiscalyear = fiscalyear
-
-        fiscalyear = None
-        if youngest_fiscalyear:
-            fiscalyears = FiscalYear.search([
-                ('end_date', '<=', youngest_fiscalyear.start_date),
-                ('company', '=', youngest_fiscalyear.company),
-                ], order=[('end_date', 'DESC')], limit=1)
-            if fiscalyears:
-                fiscalyear = fiscalyears[0]
-
-        if fiscalyear:
-            if fiscalyear.state == 'close':
-                deferrals = Deferral.search([
-                    ('fiscalyear', '=', fiscalyear.id),
-                    ('account', 'in', [a.id for a in accounts]),
-                    ])
-                id2deferral = {}
-                for deferral in deferrals:
-                    id2deferral[deferral.account.id] = deferral
-
-                for account in accounts:
-                    if account.id in id2deferral:
-                        deferral = id2deferral[account.id]
-                        res[account.id] += deferral.debit - deferral.credit
-            else:
-                with Transaction().set_context(fiscalyear=fiscalyear.id,
-                        date=None, periods=None):
-                    res2 = cls.get_balance(accounts, name)
-                for account in accounts:
-                    res[account.id] += res2[account.id]
-
-        for account in accounts:
-            company_id = account2company[account.id]
-            to_currency = id2company[company_id].currency
-            res[account.id] = to_currency.round(res[account.id])
-        return res
+        fiscalyears = FiscalYear.browse(fiscalyear_ids)
+        func = lambda accounts, names: \
+            {names[0]: cls.get_balance(accounts, names[0])}
+        return cls._cumulate(fiscalyears, accounts, {name: balances},
+            func)[name]
 
     @classmethod
     def get_credit_debit(cls, accounts, names):
@@ -726,100 +689,111 @@ class Account(ModelSQL, ModelView):
         If cumulate is set in the context, it is the cumulate amount over all
         previous fiscal year.
         '''
-        res = {}
         pool = Pool()
         MoveLine = pool.get('account.move.line')
         FiscalYear = pool.get('account.fiscalyear')
-        Deferral = pool.get('account.account.deferral')
         cursor = Transaction().cursor
+        in_max = cursor.IN_MAX
 
+        result = {}
+        ids = [a.id for a in accounts]
         for name in names:
             if name not in ('credit', 'debit'):
                 raise Exception('Bad argument')
-            res[name] = {}
+            result[name] = dict((i, 0) for i in ids)
 
-        ids = [a.id for a in accounts]
-        line_query, fiscalyear_ids = MoveLine.query_get()
-        for i in range(0, len(ids), cursor.IN_MAX):
-            sub_ids = ids[i:i + cursor.IN_MAX]
-            red_sql, red_ids = reduce_ids('a.id', sub_ids)
-            cursor.execute('SELECT a.id, ' +
-                        ','.join('SUM(COALESCE(l.' + name + ', 0))'
-                            for name in names) + ' '
-                    'FROM account_account a '
-                        'LEFT JOIN account_move_line l '
-                        'ON (a.id = l.account) '
-                    'WHERE a.kind != \'view\' '
-                        'AND ' + red_sql + ' '
-                        'AND ' + line_query + ' '
-                        'AND a.active '
-                    'GROUP BY a.id', red_ids)
+        table = cls.__table__()
+        line = MoveLine.__table__()
+        line_query, fiscalyear_ids = MoveLine.query_get(line)
+        columns = [table.id]
+        for name in names:
+            columns.append(Sum(Coalesce(Column(line, name), 0)))
+        for i in range(0, len(ids), in_max):
+            sub_ids = ids[i:i + in_max]
+            red_sql = reduce_ids(table.id, sub_ids)
+            cursor.execute(*table.join(line, 'LEFT',
+                    condition=line.account == table.id
+                    ).select(*columns,
+                    where=red_sql & line_query,
+                    group_by=table.id))
             for row in cursor.fetchall():
                 account_id = row[0]
-                for i in range(len(names)):
+                for i, name in enumerate(names, 1):
                     # SQLite uses float for SUM
-                    if not isinstance(row[i + 1], Decimal):
-                        res[names[i]][account_id] = Decimal(str(row[i + 1]))
+                    if not isinstance(row[i], Decimal):
+                        result[name][account_id] = Decimal(str(row[i]))
                     else:
-                        res[names[i]][account_id] = row[i + 1]
-
-        account2company = {}
-        id2company = {}
-        for account in accounts:
-            account2company[account.id] = account.company.id
-            id2company[account.company.id] = account.company
-
+                        result[name][account_id] = row[i]
         for account in accounts:
             for name in names:
-                res[name].setdefault(account.id, Decimal('0.0'))
+                result[name][account.id] = account.company.currency.round(
+                    result[name][account.id])
 
-        if Transaction().context.get('cumulate'):
-            youngest_fiscalyear = None
-            for fiscalyear in FiscalYear.browse(fiscalyear_ids):
-                if (not youngest_fiscalyear
-                        or (youngest_fiscalyear.start_date
-                            > fiscalyear.start_date)):
-                    youngest_fiscalyear = fiscalyear
+        if not Transaction().context.get('cumulate'):
+            return result
+        else:
+            fiscalyears = FiscalYear.browse(fiscalyear_ids)
+            return cls._cumulate(fiscalyears, accounts, result,
+                cls.get_credit_debit)
 
-            fiscalyear = None
-            if youngest_fiscalyear:
-                fiscalyears = FiscalYear.search([
-                        ('end_date', '<=', youngest_fiscalyear.start_date),
-                        ('company', '=', youngest_fiscalyear.company),
-                        ], order=[('end_date', 'DESC')], limit=1)
-                if fiscalyears:
-                    fiscalyear, = fiscalyears
+    @classmethod
+    def _cumulate(cls, fiscalyears, accounts, values, func):
+        """
+        Cumulate previous fiscalyear values into values
+        func is the method to compute values
+        """
+        pool = Pool()
+        FiscalYear = pool.get('account.fiscalyear')
+        Deferral = pool.get('account.account.deferral')
+        in_max = Transaction().cursor.IN_MAX
+        names = values.keys()
 
-            if fiscalyear:
-                if fiscalyear.state == 'close':
-                    deferrals = Deferral.search([
-                        ('fiscalyear', '=', fiscalyear.id),
-                        ('account', 'in', ids),
-                        ])
-                    id2deferral = {}
-                    for deferral in deferrals:
-                        id2deferral[deferral.account.id] = deferral
+        youngest_fiscalyear = None
+        for fiscalyear in fiscalyears:
+            if (not youngest_fiscalyear
+                    or (youngest_fiscalyear.start_date
+                        > fiscalyear.start_date)):
+                youngest_fiscalyear = fiscalyear
 
-                    for account in accounts:
-                        if account.id in id2deferral:
-                            deferral = id2deferral[account.id]
-                            for name in names:
-                                res[name][account.id] += getattr(deferral,
-                                    name)
-                else:
-                    with Transaction().set_context(fiscalyear=fiscalyear.id,
-                            date=None, periods=None):
-                        res2 = cls.get_credit_debit(accounts, names)
-                    for account in accounts:
-                        for name in names:
-                            res[name][account.id] += res2[name][account.id]
+        fiscalyear = None
+        if youngest_fiscalyear:
+            fiscalyears = FiscalYear.search([
+                    ('end_date', '<=', youngest_fiscalyear.start_date),
+                    ('company', '=', youngest_fiscalyear.company),
+                    ], order=[('end_date', 'DESC')], limit=1)
+            if fiscalyears:
+                fiscalyear, = fiscalyears
 
-        for account in accounts:
-            company_id = account2company[account.id]
-            currency = id2company[company_id].currency
+        if not fiscalyear:
+            return values
+
+        if fiscalyear.state == 'close':
+            id2deferral = {}
+            ids = [a.id for a in accounts]
+            for i in range(0, len(ids), in_max):
+                sub_ids = ids[i:i + in_max]
+                deferrals = Deferral.search([
+                    ('fiscalyear', '=', fiscalyear.id),
+                    ('account', 'in', sub_ids),
+                    ])
+                for deferral in deferrals:
+                    id2deferral[deferral.account.id] = deferral
+
+            for account in accounts:
+                if account.id in id2deferral:
+                    deferral = id2deferral[account.id]
+                    for name in names:
+                        values[name][account.id] += getattr(deferral, name)
+        else:
+            with Transaction().set_context(fiscalyear=fiscalyear.id,
+                    date=None, periods=None):
+                previous_result = func(accounts, names)
             for name in names:
-                res[name][account.id] = currency.round(res[name][account.id])
-        return res
+                vals = values[name]
+                for account in accounts:
+                    vals[account.id] += previous_result[name][account.id]
+
+        return values
 
     def get_rec_name(self, name):
         if self.code:
@@ -829,12 +803,10 @@ class Account(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        accounts = cls.search([
-                ('code',) + clause[1:],
-                ], limit=1)
-        if accounts:
-            return [('code',) + clause[1:]]
-        return [(cls._rec_name,) + clause[1:]]
+        return ['OR',
+            ('code',) + tuple(clause[1:]),
+            (cls._rec_name,) + tuple(clause[1:]),
+            ]
 
     @classmethod
     def copy(cls, accounts, default=None):
@@ -842,24 +814,30 @@ class Account(ModelSQL, ModelView):
             default = {}
         default['left'] = 0
         default['right'] = 0
+        default.setdefault('template')
         default.setdefault('deferrals', [])
         new_accounts = super(Account, cls).copy(accounts, default=default)
         cls._rebuild_tree('parent', None, 0)
         return new_accounts
 
     @classmethod
-    def write(cls, accounts, vals):
-        if not vals.get('active', True):
-            MoveLine = Pool().get('account.move.line')
-            childs = cls.search([
-                    ('parent', 'child_of', [a.id for a in accounts]),
-                    ])
-            if MoveLine.search([
-                        ('account', 'in', [a.id for a in childs]),
-                        ]):
-                vals = vals.copy()
-                del vals['active']
-        super(Account, cls).write(accounts, vals)
+    def write(cls, *args):
+        pool = Pool()
+        MoveLine = pool.get('account.move.line')
+        actions = iter(args)
+        args = []
+        for accounts, values in zip(actions, actions):
+            if not values.get('active', True):
+                childs = cls.search([
+                        ('parent', 'child_of', [a.id for a in accounts]),
+                        ])
+                if MoveLine.search([
+                            ('account', 'in', [a.id for a in childs]),
+                            ]):
+                    values = values.copy()
+                    del values['active']
+            args.extend((accounts, values))
+        super(Account, cls).write(*args)
 
     @classmethod
     def delete(cls, accounts):
@@ -978,6 +956,9 @@ class AccountDeferral(ModelSQL, ModelView):
         required=True, depends=['currency_digits'])
     credit = fields.Numeric('Credit', digits=(16, Eval('currency_digits', 2)),
         required=True, depends=['currency_digits'])
+    balance = fields.Function(fields.Numeric('Balance',
+            digits=(16, Eval('currency_digits', 2)),
+            depends=['currency_digits']), 'get_balance')
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
             'get_currency_digits')
 
@@ -992,6 +973,9 @@ class AccountDeferral(ModelSQL, ModelView):
             'write_deferral': 'You can not modify Account Deferral records',
             })
 
+    def get_balance(self, name):
+        return self.debit - self.credit
+
     def get_currency_digits(self, name):
         return self.account.currency_digits
 
@@ -1000,14 +984,13 @@ class AccountDeferral(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        deferrals = cls.search(['OR',
-                ('account.rec_name',) + clause[1:],
-                ('fiscalyear.rec_name',) + clause[1:],
-                ])
-        return [('id', 'in', [d.id for d in deferrals])]
+        return ['OR',
+            ('account.rec_name',) + tuple(clause[1:]),
+            ('fiscalyear.rec_name',) + tuple(clause[1:]),
+            ]
 
     @classmethod
-    def write(cls, deferrals, vals):
+    def write(cls, deferrals, values, *args):
         cls.raise_user_error('write_deferral')
 
 
@@ -1016,7 +999,7 @@ class OpenChartAccountStart(ModelView):
     __name__ = 'account.open_chart.start'
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
             help='Leave empty for all open fiscal year')
-    posted = fields.Boolean('Posted Move', help='Show only posted move')
+    posted = fields.Boolean('Posted Moves', help='Show posted moves only')
 
     @staticmethod
     def default_posted():
@@ -1053,7 +1036,7 @@ class PrintGeneralLedgerStart(ModelView):
     'Print General Ledger'
     __name__ = 'account.print_general_ledger.start'
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
-            required=True, on_change=['fiscalyear'])
+        required=True)
     start_period = fields.Many2One('account.period', 'Start Period',
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
@@ -1088,6 +1071,7 @@ class PrintGeneralLedgerStart(ModelView):
     def default_empty_account():
         return False
 
+    @fields.depends('fiscalyear')
     def on_change_fiscalyear(self):
         return {
             'start_period': None,
@@ -1195,10 +1179,10 @@ class GeneralLedger(Report):
         localcontext['end_period'] = periods[-1]
 
         if not data['empty_account']:
-            account2lines = cls.get_lines(accounts,
-                end_periods, data['posted'])
-            for account in (set(accounts) - set(account2lines)):
-                accounts.remove(account)
+            account2lines = dict(cls.get_lines(accounts,
+                    end_periods, data['posted']))
+            accounts = Account.browse([a.id for a in accounts
+                    if a in account2lines])
 
         account_id2lines = cls.lines(accounts,
             list(set(end_periods).difference(set(start_periods))),
@@ -1224,11 +1208,14 @@ class GeneralLedger(Report):
             ]
         if posted:
             clause.append(('move.state', '=', 'posted'))
-        moves = MoveLine.search(clause, order=[])
-        res = {}
-        for move in moves:
-            res.setdefault(move.account, []).append(move)
-        return res
+        lines = MoveLine.search(clause,
+            order=[
+                ('account', 'ASC'),  # TODO replace by account.id
+                ('date', 'ASC'),
+                ])
+        key = operator.attrgetter('account')
+        lines.sort(key=key)
+        return groupby(lines, key)
 
     @classmethod
     def lines(cls, accounts, periods, posted):
@@ -1239,8 +1226,7 @@ class GeneralLedger(Report):
         state_selections = dict(Move.fields_get(
                 fields_names=['state'])['state']['selection'])
 
-        for account, lines in account2lines.iteritems():
-            lines.sort(lambda x, y: cmp(x.date, y.date))
+        for account, lines in account2lines:
             balance = Decimal('0.0')
             for line in lines:
                 balance += line.debit - line.credit
@@ -1265,8 +1251,7 @@ class PrintTrialBalanceStart(ModelView):
     'Print Trial Balance'
     __name__ = 'account.print_trial_balance.start'
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
-            required=True, on_change=['fiscalyear'],
-            depends=['start_period', 'end_period'])
+        required=True)
     start_period = fields.Many2One('account.period', 'Start Period',
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
@@ -1302,6 +1287,7 @@ class PrintTrialBalanceStart(ModelView):
     def default_empty_account():
         return False
 
+    @fields.depends('fiscalyear')
     def on_change_fiscalyear(self):
         return {
             'start_period': None,
@@ -1405,17 +1391,17 @@ class TrialBalance(Report):
                 posted=data['posted']):
             end_accounts = Account.browse(accounts)
 
-        to_remove = []
+        to_remove = set()
         if not data['empty_account']:
             for account in in_accounts:
                 if account.debit == Decimal('0.0') \
                         and account.credit == Decimal('0.0'):
-                    to_remove.append(account.id)
+                    to_remove.add(account)
 
         accounts = []
         for start_account, in_account, end_account in izip(
                 start_accounts, in_accounts, end_accounts):
-            if in_account.id in to_remove:
+            if in_account in to_remove:
                 continue
             accounts.append({
                     'code': start_account.code,
@@ -1496,6 +1482,7 @@ class OpenBalanceSheet(Wizard):
                     self.start.date.day),
                 'posted': self.start.posted,
                 'company': company.id,
+                'cumulate': True,
                 })
         action['name'] += ' %s - %s' % (date, company.rec_name)
         return action, {}
@@ -1508,8 +1495,7 @@ class OpenIncomeStatementStart(ModelView):
     'Open Income Statement'
     __name__ = 'account.open_income_statement.start'
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
-            required=True, on_change=['fiscalyear'],
-            depends=['start_period', 'end_period'])
+        required=True)
     start_period = fields.Many2One('account.period', 'Start Period',
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
@@ -1539,11 +1525,12 @@ class OpenIncomeStatementStart(ModelView):
     def default_posted():
         return False
 
+    @fields.depends('fiscalyear')
     def on_change_fiscalyear(self):
         return {
             'start_period': None,
             'end_period': None,
-        }
+            }
 
 
 class OpenIncomeStatement(Wizard):
@@ -1956,37 +1943,39 @@ class ThirdPartyBalance(Report):
         pool = Pool()
         Party = pool.get('party.party')
         MoveLine = pool.get('account.move.line')
+        Move = pool.get('account.move')
+        Account = pool.get('account.account')
         Company = pool.get('company.company')
         Date = pool.get('ir.date')
         cursor = Transaction().cursor
 
+        line = MoveLine.__table__()
+        move = Move.__table__()
+        account = Account.__table__()
+
         company = Company(data['company'])
         localcontext['company'] = company
         localcontext['digits'] = company.currency.digits
         localcontext['fiscalyear'] = data['fiscalyear']
         with Transaction().set_context(context=localcontext):
-            line_query, _ = MoveLine.query_get()
+            line_query, _ = MoveLine.query_get(line)
         if data['posted']:
-            posted_clause = "AND m.state = 'posted' "
+            posted_clause = move.state == 'posted'
         else:
-            posted_clause = ""
+            posted_clause = Literal(True)
 
-        cursor.execute('SELECT l.party, SUM(l.debit), SUM(l.credit) '
-                'FROM account_move_line l '
-                    'JOIN account_move m ON (l.move = m.id) '
-                    'JOIN account_account a ON (l.account = a.id) '
-                'WHERE l.party IS NOT NULL '
-                    'AND a.active '
-                    'AND a.kind IN (\'payable\',\'receivable\') '
-                    'AND l.reconciliation IS NULL '
-                    'AND a.company = %s '
-                    'AND (l.maturity_date <= %s '
-                        'OR l.maturity_date IS NULL) '
-                    'AND ' + line_query + ' '
-                    + posted_clause +
-                'GROUP BY l.party '
-                'HAVING (SUM(l.debit) != 0 OR SUM(l.credit) != 0)',
-                (data['company'], Date.today()))
+        cursor.execute(*line.join(move, condition=line.move == move.id
+                ).join(account, condition=line.account == account.id
+                ).select(line.party, Sum(line.debit), Sum(line.credit),
+                where=(line.party != None)
+                & account.active
+                & account.kind.in_(('payable', 'receivable'))
+                & (account.company == data['company'])
+                & ((line.maturity_date <= Date.today())
+                    | (line.maturity_date == None))
+                & line_query & posted_clause,
+                group_by=line.party,
+                having=(Sum(line.debit) != 0) | (Sum(line.credit) != 0)))
 
         res = cursor.fetchall()
         id2party = {}
@@ -2095,15 +2084,21 @@ class AgedBalance(Report):
         pool = Pool()
         Party = pool.get('party.party')
         MoveLine = pool.get('account.move.line')
+        Move = pool.get('account.move')
+        Account = pool.get('account.account')
         Company = pool.get('company.company')
         Date = pool.get('ir.date')
         cursor = Transaction().cursor
 
+        line = MoveLine.__table__()
+        move = Move.__table__()
+        account = Account.__table__()
+
         company = Company(data['company'])
         localcontext['digits'] = company.currency.digits
         localcontext['posted'] = data['posted']
         with Transaction().set_context(context=localcontext):
-            line_query, _ = MoveLine.query_get()
+            line_query, _ = MoveLine.query_get(line)
 
         terms = (data['term1'], data['term2'], data['term3'])
         if data['unit'] == 'month':
@@ -2119,30 +2114,22 @@ class AgedBalance(Report):
 
         res = {}
         for position, term in enumerate(terms):
-            if position == 2:
-                term_query = 'l.maturity_date <= %s'
-                term_args = (Date.today() - term * coef,)
-            else:
-                term_query = '(l.maturity_date <= %s '\
-                    'AND l.maturity_date > %s) '
-                term_args = (
-                    Date.today() - term * coef,
+            term_query = line.maturity_date <= (Date.today() - term * coef)
+            if position != 2:
+                term_query &= line.maturity_date > (
                     Date.today() - terms[position + 1] * coef)
 
-            cursor.execute('SELECT l.party, SUM(l.debit) - SUM(l.credit) '
-                'FROM account_move_line l '
-                'JOIN account_move m ON (l.move = m.id) '
-                'JOIN account_account a ON (l.account = a.id) '
-                'WHERE l.party IS NOT NULL '
-                    'AND a.active '
-                    'AND a.kind IN (' + ','.join(('%s',) * len(kind)) + ") "
-                    'AND l.reconciliation IS NULL '
-                    'AND a.company = %s '
-                    'AND ' + term_query + ' '
-                    'AND ' + line_query + ' '
-                    'GROUP BY l.party '
-                    'HAVING (SUM(l.debit) - SUM(l.credit) != 0)',
-                kind + (data['company'],) + term_args)
+            cursor.execute(*line.join(move, condition=line.move == move.id
+                    ).join(account, condition=line.account == account.id
+                    ).select(line.party, Sum(line.debit) - Sum(line.credit),
+                    where=(line.party != None)
+                    & account.active
+                    & account.kind.in_(kind)
+                    & (line.reconciliation == None)
+                    & (account.company == data['company'])
+                    & term_query & line_query,
+                    group_by=line.party,
+                    having=(Sum(line.debit) - Sum(line.credit)) != 0))
             for party, solde in cursor.fetchall():
                 if party in res:
                     res[party][position] = solde
