@@ -1,13 +1,15 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+import datetime
 from decimal import Decimal
+from sql.aggregate import Sum
+
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard, StateView, StateAction, Button
-from trytond.backend import TableHandler
+from trytond import backend
 from trytond.pyson import Eval, If, Bool, PYSONEncoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
-from trytond.config import CONFIG
 
 __all__ = ['TaxGroup', 'TaxCodeTemplate', 'TaxCode',
     'OpenChartTaxCodeStart', 'OpenChartTaxCode',
@@ -17,15 +19,23 @@ __all__ = ['TaxGroup', 'TaxCodeTemplate', 'TaxCode',
     'AccountTemplateTaxTemplate', 'AccountTemplate2', 'AccountTax', 'Account2']
 __metaclass__ = PoolMeta
 
+KINDS = [
+    ('sale', 'Sale'),
+    ('purchase', 'Purchase'),
+    ('both', 'Both'),
+    ]
+
 
 class TaxGroup(ModelSQL, ModelView):
     'Tax Group'
     __name__ = 'account.tax.group'
     name = fields.Char('Name', size=None, required=True, translate=True)
     code = fields.Char('Code', size=None, required=True)
+    kind = fields.Selection(KINDS, 'Kind', required=True)
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         super(TaxGroup, cls).__register__(module_name)
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
@@ -33,6 +43,10 @@ class TaxGroup(ModelSQL, ModelView):
         # Migration from 1.4 drop code_uniq constraint
         table.drop_constraint('code_uniq')
 
+    @staticmethod
+    def default_kind():
+        return 'both'
+
 
 class TaxCodeTemplate(ModelSQL, ModelView):
     'Tax Code Template'
@@ -48,16 +62,14 @@ class TaxCodeTemplate(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(TaxCodeTemplate, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_tax_code'),
-            ]
-        cls._error_messages.update({
-                'recursive_tax_code': \
-                    'You can not create recursive tax codes!',
-                })
         cls._order.insert(0, ('code', 'ASC'))
         cls._order.insert(0, ('account', 'ASC'))
 
+    @classmethod
+    def validate(cls, templates):
+        super(TaxCodeTemplate, cls).validate(templates)
+        cls.check_recursion(templates)
+
     def _get_tax_code_value(self, code=None):
         '''
         Set values for tax code creation.
@@ -85,6 +97,7 @@ class TaxCodeTemplate(ModelSQL, ModelView):
         pool = Pool()
         TaxCode = pool.get('account.tax.code')
         Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2tax_code is None:
             template2tax_code = {}
@@ -94,9 +107,9 @@ class TaxCodeTemplate(ModelSQL, ModelView):
             vals['company'] = company_id
             vals['parent'] = parent_id
 
-            new_tax_code = TaxCode.create(vals)
+            new_tax_code, = TaxCode.create([vals])
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -137,8 +150,8 @@ class TaxCode(ModelSQL, ModelView):
             domain=[('company', '=', Eval('company', 0))], depends=['company'])
     childs = fields.One2Many('account.tax.code', 'parent', 'Children',
             domain=[('company', '=', Eval('company', 0))], depends=['company'])
-    currency_digits = fields.Function(fields.Integer('Currency Digits',
-        on_change_with=['company']), 'on_change_with_currency_digits')
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+        'on_change_with_currency_digits')
     sum = fields.Function(fields.Numeric('Sum',
         digits=(16, Eval('currency_digits', 2)), depends=['currency_digits']),
         'get_sum')
@@ -148,15 +161,13 @@ class TaxCode(ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(TaxCode, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_tax_code'),
-            ]
-        cls._error_messages.update({
-                'recursive_tax_code': \
-                    'You can not create recursive tax codes!',
-                })
         cls._order.insert(0, ('code', 'ASC'))
 
+    @classmethod
+    def validate(cls, codes):
+        super(TaxCode, cls).validate(codes)
+        cls.check_recursion(codes)
+
     @staticmethod
     def default_active():
         return True
@@ -165,6 +176,7 @@ class TaxCode(ModelSQL, ModelView):
     def default_company():
         return Transaction().context.get('company')
 
+    @fields.depends('company')
     def on_change_with_currency_digits(self, name=None):
         if self.company:
             return self.company.currency.digits
@@ -176,23 +188,23 @@ class TaxCode(ModelSQL, ModelView):
         res = {}
         pool = Pool()
         MoveLine = pool.get('account.move.line')
+        TaxLine = pool.get('account.tax.line')
+
+        code = cls.__table__()
+        tax_line = TaxLine.__table__()
+        move_line = MoveLine.__table__()
 
         childs = cls.search([
                 ('parent', 'child_of', [c.id for c in codes]),
                 ])
         all_codes = list(set(codes) | set(childs))
-        line_query, _ = MoveLine.query_get()
-        cursor.execute('SELECT c.id, SUM(tl.amount) '
-            'FROM account_tax_code c, '
-                'account_tax_line tl, '
-                'account_move_line l '
-            'WHERE c.id = tl.code '
-                'AND tl.move_line = l.id '
-                'AND c.id IN (' +
-                    ','.join(('%s',) * len(all_codes)) + ') '
-                'AND ' + line_query + ' '
-                'AND c.active '
-            'GROUP BY c.id', [c.id for c in all_codes])
+        line_query, _ = MoveLine.query_get(move_line)
+        cursor.execute(*code.join(tax_line, condition=tax_line.code == code.id
+                ).join(move_line, condition=tax_line.move_line == move_line.id
+                ).select(code.id, Sum(tax_line.amount),
+                where=code.id.in_([c.id for c in all_codes])
+                & code.active & line_query,
+                group_by=code.id))
         code_sum = {}
         for code_id, sum in cursor.fetchall():
             # SQLite uses float for SUM
@@ -219,10 +231,10 @@ class TaxCode(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        codes = cls.search([('code',) + tuple(clause[1:])], limit=1, order=[])
-        if codes:
-            return [('code',) + tuple(clause[1:])]
-        return [('name',) + tuple(clause[1:])]
+        return ['OR',
+            ('code',) + tuple(clause[1:]),
+            (cls._rec_name,) + tuple(clause[1:]),
+            ]
 
     @classmethod
     def delete(cls, codes):
@@ -238,7 +250,9 @@ class TaxCode(ModelSQL, ModelView):
         tax code id as value, used to convert template id into tax code. The
         dictionary is filled with new tax codes
         '''
-        Lang = Pool().get('ir.lang')
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2tax_code is None:
             template2tax_code = {}
@@ -248,7 +262,7 @@ class TaxCode(ModelSQL, ModelView):
             if vals:
                 self.write([self], vals)
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self.template._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -309,7 +323,8 @@ class OpenChartTaxCode(Wizard):
     def do_open_(self, action):
         if self.start.method == 'fiscalyear':
             action['pyson_context'] = PYSONEncoder().encode({
-                    'fiscalyear': self.start.fiscalyear.id,
+                    'fiscalyear': (self.start.fiscalyear.id
+                        if self.start.fiscalyear else None),
                     })
             if self.start.fiscalyear:
                 action['name'] += ' - %s' % self.start.fiscalyear.rec_name
@@ -332,11 +347,20 @@ class TaxTemplate(ModelSQL, ModelView):
     name = fields.Char('Name', required=True, translate=True)
     description = fields.Char('Description', required=True, translate=True)
     group = fields.Many2One('account.tax.group', 'Group')
-    sequence = fields.Integer('Sequence',
-        order_field='(%(table)s.sequence IS NULL) %(order)s, '
-        '%(table)s.sequence %(order)s')
-    amount = fields.Numeric('Amount', digits=(16, 8))
-    percentage = fields.Numeric('Percentage', digits=(16, 8))
+    sequence = fields.Integer('Sequence')
+    start_date = fields.Date('Starting Date')
+    end_date = fields.Date('Ending Date')
+    amount = fields.Numeric('Amount', digits=(16, 8),
+        states={
+            'required': Eval('type') == 'fixed',
+            'invisible': Eval('type') != 'fixed',
+            },
+        depends=['type'])
+    rate = fields.Numeric('Rate', digits=(14, 10),
+        states={
+            'required': Eval('type') == 'percentage',
+            'invisible': Eval('type') != 'percentage',
+            }, depends=['type'])
     type = fields.Selection([
         ('percentage', 'Percentage'),
         ('fixed', 'Fixed'),
@@ -373,6 +397,7 @@ class TaxTemplate(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         super(TaxTemplate, cls).__register__(module_name)
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
@@ -383,6 +408,19 @@ class TaxTemplate(ModelSQL, ModelView):
         #Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
+        # Migration from 2.8: rename percentage into rate
+        if table.column_exist('percentage'):
+            sql_table = cls.__table__()
+            cursor.execute(*sql_table.update(
+                    columns=[sql_table.rate],
+                    values=[sql_table.percentage / 100]))
+            table.drop_column('percentage')
+
+    @staticmethod
+    def order_sequence(tables):
+        table, _ = tables[None]
+        return [table.sequence == None, table.sequence]
+
     @staticmethod
     def default_type():
         return 'percentage'
@@ -413,8 +451,9 @@ class TaxTemplate(ModelSQL, ModelView):
         '''
         res = {}
         for field in ('name', 'description', 'sequence', 'amount',
-                'percentage', 'type', 'invoice_base_sign', 'invoice_tax_sign',
-                'credit_note_base_sign', 'credit_note_tax_sign'):
+                'rate', 'type', 'invoice_base_sign', 'invoice_tax_sign',
+                'credit_note_base_sign', 'credit_note_tax_sign',
+                'start_date', 'end_date'):
             if not tax or getattr(tax, field) != getattr(self, field):
                 res[field] = getattr(self, field)
         for field in ('group',):
@@ -446,6 +485,7 @@ class TaxTemplate(ModelSQL, ModelView):
         pool = Pool()
         Tax = pool.get('account.tax')
         Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2tax is None:
             template2tax = {}
@@ -456,38 +496,38 @@ class TaxTemplate(ModelSQL, ModelView):
             vals['parent'] = parent_id
             if self.invoice_account:
                 vals['invoice_account'] = \
-                        template2account[self.invoice_account.id]
+                    template2account[self.invoice_account.id]
             else:
                 vals['invoice_account'] = None
             if self.credit_note_account:
                 vals['credit_note_account'] = \
-                        template2account[self.credit_note_account.id]
+                    template2account[self.credit_note_account.id]
             else:
                 vals['credit_note_account'] = None
             if self.invoice_base_code:
                 vals['invoice_base_code'] = \
-                        template2tax_code[self.invoice_base_code.id]
+                    template2tax_code[self.invoice_base_code.id]
             else:
                 vals['invoice_base_code'] = None
             if self.invoice_tax_code:
                 vals['invoice_tax_code'] = \
-                        template2tax_code[self.invoice_tax_code.id]
+                    template2tax_code[self.invoice_tax_code.id]
             else:
                 vals['invoice_tax_code'] = None
             if self.credit_note_base_code:
                 vals['credit_note_base_code'] = \
-                        template2tax_code[self.credit_note_base_code.id]
+                    template2tax_code[self.credit_note_base_code.id]
             else:
                 vals['credit_note_base_code'] = None
             if self.credit_note_tax_code:
                 vals['credit_note_tax_code'] = \
-                        template2tax_code[self.credit_note_tax_code.id]
+                    template2tax_code[self.credit_note_tax_code.id]
             else:
                 vals['credit_note_tax_code'] = None
 
-            new_tax = Tax.create(vals)
+            new_tax, = Tax.create([vals])
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -521,7 +561,7 @@ class Tax(ModelSQL, ModelView):
     Account Tax
 
     Type:
-        percentage: tax = price * amount
+        percentage: tax = price * rate
         fixed: tax = amount
         none: tax = none
     '''
@@ -534,23 +574,22 @@ class Tax(ModelSQL, ModelView):
                 'invisible': Bool(Eval('parent')),
             }, depends=['parent'])
     active = fields.Boolean('Active')
-    sequence = fields.Integer('Sequence',
-        order_field='(%(table)s.sequence IS NULL) %(order)s, '
-        '%(table)s.sequence %(order)s',
-        help='Use to order the taxes')
-    currency_digits = fields.Function(fields.Integer('Currency Digits',
-        on_change_with=['company']), 'on_change_with_currency_digits')
+    sequence = fields.Integer('Sequence', help='Use to order the taxes')
+    start_date = fields.Date('Starting Date')
+    end_date = fields.Date('Ending Date')
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+        'on_change_with_currency_digits')
     amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
         states={
             'required': Eval('type') == 'fixed',
             'invisible': Eval('type') != 'fixed',
             }, help='In company\'s currency',
         depends=['type', 'currency_digits'])
-    percentage = fields.Numeric('Percentage', digits=(16, 8),
+    rate = fields.Numeric('Rate', digits=(14, 10),
         states={
             'required': Eval('type') == 'percentage',
             'invisible': Eval('type') != 'percentage',
-            }, help='In %', depends=['type'])
+            }, depends=['type'])
     type = fields.Selection([
         ('percentage', 'Percentage'),
         ('fixed', 'Fixed'),
@@ -561,7 +600,7 @@ class Tax(ModelSQL, ModelView):
     company = fields.Many2One('company.company', 'Company', required=True,
         domain=[
             ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', 0)),
+                Eval('context', {}).get('company', -1)),
             ], select=True)
     invoice_account = fields.Many2One('account.account', 'Invoice Account',
         domain=[
@@ -637,6 +676,7 @@ class Tax(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         super(Tax, cls).__register__(module_name)
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
@@ -647,6 +687,19 @@ class Tax(ModelSQL, ModelView):
         # Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
+        # Migration from 2.8: rename percentage into rate
+        if table.column_exist('percentage'):
+            sql_table = cls.__table__()
+            cursor.execute(*sql_table.update(
+                    columns=[sql_table.rate],
+                    values=[sql_table.percentage / 100]))
+            table.drop_column('percentage')
+
+    @staticmethod
+    def order_sequence(tables):
+        table, _ = tables[None]
+        return [table.sequence == None, table.sequence]
+
     @staticmethod
     def default_active():
         return True
@@ -679,6 +732,7 @@ class Tax(ModelSQL, ModelView):
     def default_company():
         return Transaction().context.get('company')
 
+    @fields.depends('company')
     def on_change_with_currency_digits(self, name=None):
         if self.company:
             return self.company.currency.digits
@@ -686,7 +740,7 @@ class Tax(ModelSQL, ModelView):
 
     def _process_tax(self, price_unit):
         if self.type == 'percentage':
-            amount = price_unit * self.percentage / Decimal('100')
+            amount = price_unit * self.rate
             return {
                 'base': price_unit,
                 'amount': amount,
@@ -701,13 +755,17 @@ class Tax(ModelSQL, ModelView):
                 }
 
     @classmethod
-    def _unit_compute(cls, taxes, price_unit):
+    def _unit_compute(cls, taxes, price_unit, date):
         res = []
         for tax in taxes:
+            start_date = tax.start_date or datetime.date.min
+            end_date = tax.end_date or datetime.date.max
+            if not (start_date <= date <= end_date):
+                continue
             if tax.type != 'none':
                 res.append(tax._process_tax(price_unit))
             if len(tax.childs):
-                res.extend(cls._unit_compute(tax.childs, price_unit))
+                res.extend(cls._unit_compute(tax.childs, price_unit, date))
         return res
 
     @classmethod
@@ -718,16 +776,20 @@ class Tax(ModelSQL, ModelView):
         return sorted(taxes, key=lambda t: (t.sequence, t.id))
 
     @classmethod
-    def compute(cls, taxes, price_unit, quantity):
+    def compute(cls, taxes, price_unit, quantity, date=None):
         '''
-        Compute taxes for price_unit and quantity.
+        Compute taxes for price_unit and quantity at the date.
         Return list of dict for each taxes and their childs with:
             base
             amount
             tax
         '''
+        pool = Pool()
+        Date = pool.get('ir.date')
+        if date is None:
+            date = Date.today()
         taxes = cls.sort_taxes(taxes)
-        res = cls._unit_compute(taxes, price_unit)
+        res = cls._unit_compute(taxes, price_unit, date)
         quantity = Decimal(str(quantity or 0.0))
         for row in res:
             row['base'] *= quantity
@@ -747,55 +809,69 @@ class Tax(ModelSQL, ModelView):
         value, used to convert template id into tax.  The dictionary is filled
         with new taxes.
         '''
-        Lang = Pool().get('ir.lang')
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2tax is None:
             template2tax = {}
 
         if self.template:
             vals = self.template._get_tax_value(tax=self)
-            if (self.template.invoice_account
-                    and self.invoice_account.id != template2account.get(
-                        self.template.invoice_account.id)):
+            invoice_account_id = (self.invoice_account.id
+                if self.invoice_account else None)
+            if (self.template.invoice_account and
+                    invoice_account_id != template2account.get(
+                            self.template.invoice_account.id)):
                 vals['invoice_account'] = template2account.get(
                     self.template.invoice_account.id)
             elif (not self.template.invoice_account
                     and self.invoice_account):
                 vals['invoice_account'] = None
-            if (self.template.credit_note_account
-                    and self.credit_note_account.id != template2account.get(
+            credit_note_account_id = (self.credit_note_account.id
+                if self.credit_note_account else None)
+            if (self.template.credit_note_account and
+                    credit_note_account_id != template2account.get(
                         self.template.credit_note_account.id)):
                 vals['credit_note_account'] = template2account.get(
                     self.template.credit_note_account.id)
             elif (not self.template.credit_note_account
                     and self.credit_note_account):
                 vals['credit_note_account'] = None
-            if (self.template.invoice_base_code
-                    and self.invoice_base_code.id != template2tax_code.get(
-                        self.template.invoice_base_code.id)):
+            invoice_base_code_id = (self.invoice_base_code.id
+                if self.invoice_base_code else None)
+            if (self.template.invoice_base_code and
+                    invoice_base_code_id != template2tax_code.get(
+                            self.template.invoice_base_code.id)):
                 vals['invoice_base_code'] = template2tax_code.get(
                     self.template.invoice_base_code.id)
             elif (not self.template.invoice_base_code
                     and self.invoice_base_code):
                 vals['invoice_base_code'] = None
+            invoice_tax_code_id = (self.invoice_tax_code.id
+                if self.invoice_tax_code else None)
             if (self.template.invoice_tax_code
-                    and self.invoice_tax_code.id != template2tax_code.get(
+                    and invoice_tax_code_id != template2tax_code.get(
                         self.template.invoice_tax_code.id)):
                 vals['invoice_tax_code'] = template2tax_code.get(
                     self.template.invoice_tax_code.id)
             elif (not self.template.invoice_tax_code
                     and self.invoice_tax_code):
                 vals['invoice_tax_code'] = None
+            credit_note_base_code_id = (self.credit_note_base_code.id
+                if self.credit_note_base_code else None)
             if (self.template.credit_note_base_code
-                    and self.credit_note_base_code.id != template2tax_code.get(
+                    and credit_note_base_code_id != template2tax_code.get(
                         self.template.credit_note_base_code.id)):
                 vals['credit_note_base_code'] = template2tax_code.get(
                     self.template.credit_note_base_code.id)
             elif (not self.template.credit_note_base_code
                     and self.credit_note_base_code):
                 vals['credit_note_base_code'] = None
+            credit_note_tax_code_id = (self.credit_note_tax_code.id
+                if self.credit_note_tax_code else None)
             if (self.template.credit_note_tax_code
-                    and self.credit_note_tax_code.id != template2tax_code.get(
+                    and credit_note_tax_code_id != template2tax_code.get(
                         self.template.credit_note_tax_code.id)):
                 vals['credit_note_tax_code'] = template2tax_code.get(
                     self.template.credit_note_tax_code.id)
@@ -806,7 +882,7 @@ class Tax(ModelSQL, ModelView):
             if vals:
                 self.write([self], vals)
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self.template._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -836,22 +912,24 @@ class TaxLine(ModelSQL, ModelView):
     'Tax Line'
     __name__ = 'account.tax.line'
     _rec_name = 'amount'
-    currency_digits = fields.Function(fields.Integer('Currency Digits',
-        on_change_with=['move_line']), 'on_change_with_currency_digits')
+    currency_digits = fields.Function(fields.Integer('Currency Digits'),
+        'on_change_with_currency_digits')
     amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
         required=True, depends=['currency_digits'])
     code = fields.Many2One('account.tax.code', 'Code', select=True,
         required=True)
     tax = fields.Many2One('account.tax', 'Tax', select=True,
-        ondelete='RESTRICT', on_change=['tax'])
+        ondelete='RESTRICT')
     move_line = fields.Many2One('account.move.line', 'Move Line',
             required=True, select=True, ondelete='CASCADE')
 
+    @fields.depends('move_line')
     def on_change_with_currency_digits(self, name=None):
         if self.move_line:
             return self.move_line.currency_digits
         return 2
 
+    @fields.depends('tax')
     def on_change_tax(self):
         return {
             'code': None,
@@ -862,10 +940,15 @@ class TaxRuleTemplate(ModelSQL, ModelView):
     'Tax Rule Template'
     __name__ = 'account.tax.rule.template'
     name = fields.Char('Name', required=True, translate=True)
+    kind = fields.Selection(KINDS, 'Kind', required=True)
     lines = fields.One2Many('account.tax.rule.line.template', 'rule', 'Lines')
     account = fields.Many2One('account.account.template', 'Account Template',
             domain=[('parent', '=', None)], required=True)
 
+    @staticmethod
+    def default_kind():
+        return 'both'
+
     def _get_tax_rule_value(self, rule=None):
         '''
         Set values for tax rule creation.
@@ -873,6 +956,8 @@ class TaxRuleTemplate(ModelSQL, ModelView):
         res = {}
         if not rule or rule.name != self.name:
             res['name'] = self.name
+        if not rule or rule.kind != self.kind:
+            res['kind'] = self.kind
         if not rule or rule.template.id != self.id:
             res['template'] = self.id
         return res
@@ -888,6 +973,7 @@ class TaxRuleTemplate(ModelSQL, ModelView):
         pool = Pool()
         Rule = pool.get('account.tax.rule')
         Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2rule is None:
             template2rule = {}
@@ -895,9 +981,9 @@ class TaxRuleTemplate(ModelSQL, ModelView):
         if self.id not in template2rule:
             vals = self._get_tax_rule_value()
             vals['company'] = company_id
-            new_rule = Rule.create(vals)
+            new_rule, = Rule.create([vals])
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -923,14 +1009,19 @@ class TaxRule(ModelSQL, ModelView):
     'Tax Rule'
     __name__ = 'account.tax.rule'
     name = fields.Char('Name', required=True, translate=True)
+    kind = fields.Selection(KINDS, 'Kind', required=True)
     company = fields.Many2One('company.company', 'Company', required=True,
         select=True, domain=[
             ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', 0)),
+                Eval('context', {}).get('company', -1)),
             ])
     lines = fields.One2Many('account.tax.rule.line', 'rule', 'Lines')
     template = fields.Many2One('account.tax.rule.template', 'Template')
 
+    @staticmethod
+    def default_kind():
+        return 'both'
+
     def apply(self, tax, pattern):
         '''
         Apply rule on tax
@@ -954,7 +1045,9 @@ class TaxRule(ModelSQL, ModelView):
         rule id as value, used to convert template id into tax rule. The
         dictionary is filled with new tax rules.
         '''
-        Lang = Pool().get('ir.lang')
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        Config = pool.get('ir.configuration')
 
         if template2rule is None:
             template2rule = {}
@@ -964,7 +1057,7 @@ class TaxRule(ModelSQL, ModelView):
             if vals:
                 self.write([self], vals)
 
-            prev_lang = self._context.get('language') or CONFIG['language']
+            prev_lang = self._context.get('language') or Config.get_language()
             prev_data = {}
             for field_name, field in self.template._fields.iteritems():
                 if getattr(field, 'translate', False):
@@ -995,20 +1088,36 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
     origin_tax = fields.Many2One('account.tax.template', 'Original Tax',
         domain=[
             ('account', '=', Eval('_parent_rule', {}).get('account', 0)),
-            ('group', '=', Eval('group'))
+            ('group', '=', Eval('group')),
+            ['OR',
+                ('group', '=', None),
+                If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
+                    ('group.kind', 'in', ['sale', 'both']),
+                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
+                        'purchase',
+                        ('group.kind', 'in', ['purchase', 'both']),
+                        ('group.kind', 'in', ['sale', 'purchase', 'both']))),
+                ],
             ],
-        help='If the original tax template is filled, the rule will be ' \
-            'applied only for this tax template.',
+        help=('If the original tax template is filled, the rule will be '
+            'applied only for this tax template.'),
         depends=['group'])
     tax = fields.Many2One('account.tax.template', 'Substitution Tax',
         domain=[
             ('account', '=', Eval('_parent_rule', {}).get('account', 0)),
             ('group', '=', Eval('group')),
+            ['OR',
+                ('group', '=', None),
+                If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
+                    ('group.kind', 'in', ['sale', 'both']),
+                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
+                        'purchase',
+                        ('group.kind', 'in', ['purchase', 'both']),
+                        ('group.kind', 'in', ['sale', 'purchase', 'both']))),
+                ],
             ],
         depends=['group'])
-    sequence = fields.Integer('Sequence',
-        order_field='(%(table)s.sequence IS NULL) %(order)s, '
-        '%(table)s.sequence %(order)s')
+    sequence = fields.Integer('Sequence')
 
     @classmethod
     def __setup__(cls):
@@ -1018,6 +1127,7 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
 
@@ -1026,6 +1136,11 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
         # Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
+    @staticmethod
+    def order_sequence(tables):
+        table, _ = tables[None]
+        return [table.sequence == None, table.sequence]
+
     def _get_tax_rule_line_value(self, rule_line=None):
         '''
         Set values for tax rule line creation.
@@ -1033,8 +1148,6 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
         res = {}
         if not rule_line or rule_line.group != self.group:
             res['group'] = self.group.id if self.group else None
-        if not rule_line or rule_line.origin_tax != self.origin_tax:
-            res['origin_tax'] = self.origin_tax.id if self.origin_tax else None
         if not rule_line or rule_line.sequence != self.sequence:
             res['sequence'] = self.sequence
         if not rule_line or rule_line.template != self:
@@ -1070,7 +1183,7 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
                 vals['tax'] = template2tax[self.tax.id]
             else:
                 vals['tax'] = None
-            new_rule_line = RuleLine.create(vals)
+            new_rule_line, = RuleLine.create([vals])
             template2rule_line[self.id] = new_rule_line.id
         return template2rule_line[self.id]
 
@@ -1086,19 +1199,35 @@ class TaxRuleLine(ModelSQL, ModelView):
         domain=[
             ('company', '=', Eval('_parent_rule', {}).get('company')),
             ('group', '=', Eval('group')),
+            ['OR',
+                ('group', '=', None),
+                If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
+                    ('group.kind', 'in', ['sale', 'both']),
+                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
+                        'purchase',
+                        ('group.kind', 'in', ['purchase', 'both']),
+                        ('group.kind', 'in', ['sale', 'purchase', 'both']))),
+                ],
             ],
-        help='If the original tax is filled, the rule will be applied ' \
-            'only for this tax.',
+        help=('If the original tax is filled, the rule will be applied '
+            'only for this tax.'),
         depends=['group'])
     tax = fields.Many2One('account.tax', 'Substitution Tax',
         domain=[
             ('company', '=', Eval('_parent_rule', {}).get('company')),
             ('group', '=', Eval('group')),
+            ['OR',
+                ('group', '=', None),
+                If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
+                    ('group.kind', 'in', ['sale', 'both']),
+                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
+                        'purchase',
+                        ('group.kind', 'in', ['purchase', 'both']),
+                        ('group.kind', 'in', ['sale', 'purchase', 'both']))),
+                ],
             ],
         depends=['group'])
-    sequence = fields.Integer('Sequence',
-        order_field='(%(table)s.sequence IS NULL) %(order)s, '
-        '%(table)s.sequence %(order)s')
+    sequence = fields.Integer('Sequence')
     template = fields.Many2One('account.tax.rule.line.template', 'Template')
 
     @classmethod
@@ -1109,6 +1238,7 @@ class TaxRuleLine(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
 
@@ -1117,6 +1247,11 @@ class TaxRuleLine(ModelSQL, ModelView):
         # Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
+    @staticmethod
+    def order_sequence(tables):
+        table, _ = tables[None]
+        return [table.sequence == None, table.sequence]
+
     def match(self, pattern):
         '''
         Match line on pattern
@@ -1129,7 +1264,8 @@ class TaxRuleLine(ModelSQL, ModelView):
             if not getattr(self, field) and field != 'group':
                 continue
             if self._fields[field]._type == 'many2one':
-                if getattr(self, field).id != pattern[field]:
+                if ((getattr(self, field).id if getattr(self, field) else None)
+                        != pattern[field]):
                     return False
             else:
                 if getattr(self, field) != pattern[field]:
@@ -1165,14 +1301,22 @@ class TaxRuleLine(ModelSQL, ModelView):
                 vals['rule'] = template2rule[self.template.rule.id]
             if self.origin_tax:
                 if self.template.origin_tax:
-                    if self.origin_tax.id != \
-                            template2tax[self.template.origin_tax.id]:
+                    if (self.origin_tax.id !=
+                            template2tax[self.template.origin_tax.id]):
                         vals['origin_tax'] = template2tax[
                             self.template.origin_tax.id]
+                else:
+                    vals['origin_tax'] = None
+            elif self.template.origin_tax:
+                vals['origin_tax'] = template2tax[self.template.origin_tax.id]
             if self.tax:
                 if self.template.tax:
                     if self.tax.id != template2tax[self.template.tax.id]:
                         vals['tax'] = template2tax[self.template.tax.id]
+                else:
+                    vals['tax'] = None
+            elif self.template.tax:
+                vals['tax'] = template2tax[self.template.tax.id]
             if vals:
                 self.write([self], vals)
             template2rule_line[self.template.id] = self.id
@@ -1254,6 +1398,6 @@ class Account2:
                 ('company', '=', Eval('company')),
                 ('parent', '=', None),
             ],
-            help='Default tax for manual encoding of move lines \n' \
-                    'for journal types: "expense" and "revenue"',
+            help=('Default tax for manual encoding of move lines \n'
+                'for journal types: "expense" and "revenue"'),
             depends=['company'])
